@@ -86,21 +86,22 @@ SETUP and CONFIGURATIONS:
 
 EXAMPLES:
 List files
-    simeon can list files on S3 for your organization based on criteria like file type (sql or log or email), time intervals (begin and end dates), and site (edx or edge or patches).
-        # List SQL files dumped since 2021-01-01
-        simeon list -s edx -o mitx -f sql -b 2021-01-01
-        # List email files dumped since 2021-01-01
-        simeon list -s edx -o mitx -f email -b 2021-01-01
-        # List tracking log files dumped since 2021-01-01
-        simeon list -s edx -o mitx -f log -b 2021-01-01
+    simeon can list files on S3 for your organization based on criteria like file type (sql or log or email), time intervals (begin and end dates),
+    and site (edx or edge or patches).
+        # List the latest SQL data dump
+        simeon list -s edx -o mitx -f sql -L
+        # List the latest email data dump
+        simeon list -s edx -o mitx -f email -L
+        # List the latest tracking log file
+        simeon list -s edx -o mitx -f log -L
 
 Download and split files
     simeon can download, decrypt and split up files into folders belonging to specific courses.
 
     o Example 1: Download, split and push SQL bundles to both GCS and BigQuery
 
-        # Download a SQL bundle with the date 2021-02-01 in its file name
-        simeon download -s edx -o mitx -f sql -b 2021-02-01 -e 2021-02-01 -d data/
+        # Download the latest SQL data dump
+        simeon download -s edx -o mitx -f sql -L -d data/
 
         # Download SQL bundles dumped any time since 2021-01-01 and
         # extract the contents for course ID MITx/12.3x/1T2021.
@@ -118,12 +119,12 @@ Download and split files
 
     o Example 2: Download, split and push tracking logs to both GCS and BigQuery
 
-        # Download a tracking log with the date 2021-02-01 in its file name
-        simeon download -s edx -o mitx -f log -b 2021-02-01 -e 2021-02-01 -d data/
+        # Download the latest tracking log file
+        simeon download -s edx -o mitx -f log -L -d data/
 
         # Download tracking logs dumped any time since 2021-01-01
         # and extract the contents for course ID MITx/12.3x/1T2021
-        # Place the download files in data/ and the output of the split in data/TRACKING_LOGS
+        # Place the downloaded files in data/ and the output of the split in data/TRACKING_LOGS
         simeon download -s edx -o mitx -c "MITx/12.3x/1T2021" -f log -b 2021-01-01 -d data -S -D data/TRACKING_LOGS/
 
         # Push to GCS the split up tracking log files inside
@@ -152,7 +153,9 @@ Make secondary/aggregated tables
 
 NOTES:
 1. Please note that SQL bundles are quite large when split up, so consider using the -c or --courses option when invoking simeon download -S or
-    simeon split to make sure that you limit the splitting to a set of course IDs. Otherwise, simeon may end up failing to complete the split operation
+    simeon split to make sure that you limit the splitting to a set of course IDs. The `--clistings-file` option is an alternative to `--courses`.
+    It expects a text file with one course ID per line.
+    If those options are not used, simeon may end up failing to complete the split operation
     due to exhausted system resources (storage to be specific).
 
 2. simeon download with file types log and email will both download and decrypt the files matching the given criteria. If the latter operations are
@@ -168,7 +171,7 @@ NOTES:
     This will help the CLI parser not confuse your positional arguments with table names (in the case of --tables) or course IDs (when --courses is used).
 
 5. Splitting tracking logs is a resource intensive process. The routine that splits the logs generates a file for each course ID encountered. If you
-    happen to have more course IDs in your logs than the running process can open operation system file descriptors, then simeon will put away records
+    happen to have more course IDs in your logs than the running process can open operating system file descriptors, then simeon will put away records
     it cannot save to disk for a second pass. Putting away the records involves using more memory than normally required. The second pass will only
     require one file descriptor at a time, so it should be safe in terms of file descriptor limits. To help simeon not have to do a second pass, you
     may increase the file descriptor limits of processes from your shell by running something like ulimit -n 2000 before calling simeon split on Unix
@@ -179,6 +182,19 @@ NOTES:
     invoked command exceeding the maximum command-line length allowed for arguments in a command. To avoid errors along those lines, please consider
     passing the positional arguments as UNIX glob patterns. For instance, simeon split --file-type log 'data/TRACKING-LOGS/*/*.log.gz' tells simeon to
     expand the given glob pattern, instead of relying on the shell to do it.
+
+7. The report subcommand relies on the presence of SQL query files to parse and send to BigQuery to execute. Any errors arising from executing the parsed
+    queries will be shown to the end user through the given log stream. While the simeon tool ships with query files for most secondary/reporting tables
+    that are based on the edx2bigquery tool, an end user should be able to point simeon to a different location with SQL query files by using
+    the --query-dir option that comes with simeon report. Additionally, these query files can contain jinja2 templated SQL code.
+    Any mentioned variables within these templated queries can be passed to simeon report by using the --extra-args option and passing key-value pair items
+    in the format var1=value1,var2=value2,var3=value3,...,varn=valuen. Further, these key-value pair items can also be typed by using the format
+    var1:i=value1,var2:s=value2,var3:f=value3,...,varn:s=valuen. In this format, the type is append to the key, separated by a colon.
+    The only supported scalar types, so far, are s for str, i for int, and f for float. If any conversion errors occur during value parsing,
+    then those are shown to the end user, and the query won't get executed. Finally, if you wish to pass an array or list to the template,
+    you will need to repeat a key multiple times. For instance, if you want to pass a list named mylist containing the integers,
+    you could write something like --extra-args mylist:i=1,mylist:i=2,mylist:i=3. This means that you'll have a python list named
+    mylist within your template, and it should contain [1, 2, 3].
 """
 CONFIGS = {
     'DEFAULT': (
@@ -189,6 +205,15 @@ CONFIGS = {
         ('file_format', configparser.ConfigParser.get),
         ('schema_dir', configparser.ConfigParser.get),
         ('max_bad_rows', configparser.ConfigParser.getint),
+        ('update_description', configparser.ConfigParser.getboolean),
+        ('schema_dir', configparser.ConfigParser.get),
+        ('query_dir', configparser.ConfigParser.get),
+        ('project', configparser.ConfigParser.get),
+        ('bucket', configparser.ConfigParser.get),
+        ('service_account_file', configparser.ConfigParser.get),
+        ('geo_table', configparser.ConfigParser.get),
+        ('youtube_table', configparser.ConfigParser.get),
+        ('extra_args', configparser.ConfigParser.get),
     ),
     'GCP': (
         ('project', configparser.ConfigParser.get),
@@ -201,13 +226,27 @@ CONFIGS = {
         ('youtube_token', configparser.ConfigParser.get),
         ('file_format', configparser.ConfigParser.get),
         ('schema_dir', configparser.ConfigParser.get),
+        ('query_dir', configparser.ConfigParser.get),
         ('max_bad_rows', configparser.ConfigParser.getint),
+        ('extra_args', configparser.ConfigParser.get),
     ),
     'AWS': (
         ('aws_cred_file', configparser.ConfigParser.get),
         ('profile_name', configparser.ConfigParser.get),
     ),
 }
+REPORT_TABLES = [
+    'video_axis', 'forum_events', 'problem_grades', 'chapter_grades',
+    'show_answer', 'video_stats_day', 'show_answer_stats_by_user',
+    'show_answer_stats_by_course', 'course_item', 'person_item',
+    'person_problem', 'course_problem', 'person_course_day',
+    'pc_video_watched', 'pc_day_totals', 'pc_day_trlang',
+    'pc_day_ip_counts', 'language_multi_transcripts', 'pc_nchapters',
+    'pc_forum', 'course_modal_language', 'course_modal_ip',
+    'forum_posts', 'forum_person', 'enrollment_events', 'enrollday_all',
+    'person_enrollment_verified', 'person_course',
+]
+EXTRA_ARG_TYPE = {'i': int, 'f': float, 's': str,}
 
 
 def parsed_date(datestr: str) -> str:
@@ -247,6 +286,8 @@ def bq_table(name):
     Check that the given BigQuery table name
     is a fully qualified one: dataset.table or project.dataset.table
     """
+    if not name:
+        return None
     chunks = name.split('.')
     if len(chunks) not in (2, 3):
         raise ArgumentTypeError(
@@ -318,6 +359,7 @@ def make_config_file(output=None):
     if output is None:
         output = os.path.join(os.path.expanduser('~'), 'simeon.cfg')
     config = configparser.ConfigParser()
+    config.optionxform = lambda s: s.lstrip('-').lower().replace('-', '_')
     config['DEFAULT'] = {
         'site': '',
         'org': '',
@@ -357,11 +399,12 @@ def find_config(fname=None, no_raise=False):
     else:
         files = [os.path.expanduser(fname)]
     config = configparser.ConfigParser()
+    config.optionxform = lambda s: s.lstrip('-').lower().replace('-', '_')
     for config_file in files:
         try:
             config.read(config_file)
         except Exception as excp:
-            if not no_raise:
+            if no_raise:
                 continue
             raise ArgumentTypeError(excp) from None
     return config
@@ -379,7 +422,10 @@ def course_listings(courses):
     """
     if courses is None:
         return None
-    return set(c.strip() for c in courses if c.strip())
+    out = set()
+    for c in courses:
+        out.add(c.strip().split(':')[-1].replace('+', '/'))
+    return out
 
 
 def courses_from_file(fname):
@@ -398,8 +444,11 @@ def courses_from_file(fname):
     if not os.path.isfile(fname):
         msg = 'The given course listings file, {f!r}, is not a valid file'
         raise ArgumentTypeError(msg.format(f=fname))
+    out = set()
     with open(fname) as fh:
-        return set(line.strip() for line in fh if line.strip())
+        for c in fh:
+            out.add(c.strip().split(':')[-1].replace('+', '/'))
+    return out
 
 
 def course_paths_from_file(fname):
@@ -431,7 +480,7 @@ def filter_generated_items(items, cdirs):
     This is used to filter the given items (which are file paths)
     using the provided course ID directories. Since the directories
     are generated by the course_paths_from_file function, the items
-    in cdirs are expected to lowercased.
+    in cdirs are expected to be lowercased.
 
     :type items: Union[List[str], Set[str]]
     :param items: Items passed to the simeon push command
@@ -497,3 +546,51 @@ def is_parallel(args):
         )
         args.is_parallel = len(expand_paths(items)) > 1
     return args.is_parallel
+
+
+def process_extra_args(extras):
+    """
+    Take a string in the form "var1=val1,var2,val2,...,varn=valn" and convert
+    it to a dict
+
+    :type extras: str
+    :param extras: String in the form "var1=val1,var2,val2,...,varn=valn"
+    :rtype: dict
+    :return: Returns a dict from the string: var for key and val for value
+    """
+    if not extras:
+        return {}
+    out = dict()
+    types = EXTRA_ARG_TYPE
+    extras = map(lambda p: p.lstrip().split('=')[:2], extras.split(','))
+    while 1:
+        try:
+            k, v = next(extras)
+            chunks = iter(k.split(':'))
+            k = next(chunks)
+            type_ = next(chunks, None)
+            func = types.get(type_) or str
+            v = func(v)
+            if k in out:
+                if not isinstance(out[k], list):
+                    out[k] = [out[k]]
+                out[k].append(v)
+            else:
+                out[k] = v
+        except StopIteration:
+            break
+        except ValueError as excp:
+            msg = (
+                'The provided extra arguments are not properly formatted: {e}'
+            )
+            raise Exception(msg.format(e=excp)) from None
+    return out
+
+
+__all__ = [
+    'bq_table', 'course_listings', 'course_paths_from_file',
+    'courses_from_file', 'expand_paths', 'filter_generated_items',
+    'find_config', 'gcs_bucket', 'is_parallel', 'items_from_files',
+    'make_config_file', 'make_logger', 'optional_file',
+    'parsed_date', 'process_extra_args',
+]
\ No newline at end of file
